﻿using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

/* License: FC */
public class ValueAddSubtract : MVRScript
{
    JSONStorableStringChooser _atomJssc;
    JSONStorableStringChooser _receiverJssc;
    JSONStorableStringChooser _receiverTargetJssc;
    JSONStorableAction _addValueAction;
    JSONStorableAction _subtractValueAction;
    JSONStorableFloat _valueToAddJsf;

    Atom _receivingAtom;
    JSONStorable _receiver;
    JSONStorableFloat _receiverTarget;

    string _receiverTargetName;
    string _missingReceiverStoreId = "";

    public override void Init()
    {
        var textField = CreateTextField(new JSONStorableString("title", "ValueAddSubtract v1.0-alpha0"));
        textField.UItext.fontSize = 36;
        textField.height = 100;
        textField.backgroundColor = Color.clear;

        _atomJssc = new JSONStorableStringChooser("Atom", SuperController.singleton.GetAtomUIDs(), null, "Atom", SyncAtom)
        {
            representsAtomUid = true,
        };
        RegisterStringChooser(_atomJssc);
        SyncAtomOptions();
        var dp = CreateFilterablePopup(_atomJssc);
        dp.popupPanelHeight = 1100f;
        dp.popup.onOpenPopupHandlers += SyncAtomOptions;

        _receiverJssc = new JSONStorableStringChooser("Receiver", null, null, "Receiver", SyncReceiver);
        RegisterStringChooser(_receiverJssc);
        dp = CreateFilterablePopup(_receiverJssc);
        dp.popupPanelHeight = 960f;

        _receiverTargetJssc = new JSONStorableStringChooser("Target", null, null, "Target", SyncReceiverTarget);
        RegisterStringChooser(_receiverTargetJssc);
        dp = CreateFilterablePopup(_receiverTargetJssc);
        dp.popupPanelHeight = 820f;

        _atomJssc.val = containingAtom.uid;

        _addValueAction = new JSONStorableAction("Add value", AddValue);
        RegisterAction(_addValueAction);
        var addValueButton = CreateButton(_addValueAction.name);
        addValueButton.button.onClick.AddListener(AddValue);

        _subtractValueAction = new JSONStorableAction("Subtract value", SubtractValue);
        RegisterAction(_subtractValueAction);
        var subtractValueButton = CreateButton(_subtractValueAction.name);
        subtractValueButton.button.onClick.AddListener(SubtractValue);

        _valueToAddJsf = new JSONStorableFloat("Value to add/subtract", 0.10f, 0.00f, 1.00f);
        RegisterFloat(_valueToAddJsf);
        CreateSlider(_valueToAddJsf);
    }

    void AddValue()
    {
        try
        {
            if(!_receiver)
            {
                SuperController.LogMessage("Select receiver");
                return;
            }

            if(_receiverTarget == null)
            {
                SuperController.LogMessage("Select target float param");
                return;
            }

            CheckMissingReceiver();
            float result = _receiverTarget.val + _valueToAddJsf.val;
            _receiverTarget.val = Mathf.Min(result, _receiverTarget.max);
        }
        catch(Exception e)
        {
            SuperController.LogError("AddValue error:" + e);
        }
    }

    void SubtractValue()
    {
        try
        {
            if(!_receiver)
            {
                SuperController.LogMessage("Select receiver");
                return;
            }

            if(_receiverTarget == null)
            {
                SuperController.LogMessage("Select target float param");
                return;
            }

            CheckMissingReceiver();
            float result = _receiverTarget.val - _valueToAddJsf.val;
            _receiverTarget.val = Mathf.Max(result, _receiverTarget.min);
        }
        catch(Exception e)
        {
            SuperController.LogError("SubtractValue error:" + e);
        }
    }

    void SyncAtomOptions()
    {
        var options = new List<string> { "Select..." };
        foreach(string atomUid in SuperController.singleton.GetAtomUIDs())
        {
            options.Add(atomUid);
        }

        _atomJssc.choices = options;
    }

    void SyncAtom(string atomUid)
    {
        var receiverChoices = new List<string> { "Select..." };
        if(atomUid != null)
        {
            _receivingAtom = SuperController.singleton.GetAtomByUid(atomUid);
            if(_receivingAtom != null)
            {
                foreach(string receiverChoice in _receivingAtom.GetStorableIDs())
                {
                    receiverChoices.Add(receiverChoice);
                }
            }
        }
        else
        {
            _receivingAtom = null;
        }

        _receiverJssc.choices = receiverChoices;
        _receiverJssc.val = "Select...";
    }

    void CheckMissingReceiver()
    {
        if(string.IsNullOrEmpty(_missingReceiverStoreId) || !_receivingAtom)
        {
            return;
        }

        var missingReceiver = _receivingAtom.GetStorableByID(_missingReceiverStoreId);
        if(missingReceiver)
        {
            string saveTargetName = _receiverTargetName;
            SyncReceiver(_missingReceiverStoreId);
            _missingReceiverStoreId = "";
            insideRestore = true;
            _receiverTargetJssc.val = saveTargetName;
            insideRestore = false;
        }
    }

    void SyncReceiver(string receiverID)
    {
        var receiverTargetChoices = new List<string> { "Select..." };
        if(_receivingAtom != null && receiverID != null)
        {
            _receiver = _receivingAtom.GetStorableByID(receiverID);
            if(_receiver != null)
            {
                foreach(string floatParam in _receiver.GetFloatParamNames())
                {
                    receiverTargetChoices.Add(floatParam);
                }
            }
            else if(receiverID != "Select...")
            {
                // some storables can be late loaded, like skin, clothing, hair, etc so must keep track of missing receiver
                _missingReceiverStoreId = receiverID;
            }
        }
        else
        {
            _receiver = null;
        }

        _receiverTargetJssc.choices = receiverTargetChoices;
        _receiverTargetJssc.val = "Select...";
    }

    void SyncReceiverTarget(string receiverTargetName)
    {
        _receiverTargetName = receiverTargetName;
        _receiverTarget = null;
        if(_receiver != null && receiverTargetName != null)
        {
            _receiverTarget = _receiver.GetFloatJSONParam(receiverTargetName);
        }
    }
}
